from math import sqrt,acos,pi
import os
import glob
import sys
from datetime import datetime
import re
import argparse

_version_ = '3.0.6'

def pf_parse_args():
    print(sys.argv, len(sys.argv))
    if len(sys.argv)<2:
        sys.argv.append('-h')
    parser = argparse.ArgumentParser(description="PathFinder. Version: "+_version_)
    parser.add_argument("-i", "--input_file", nargs='?', type=str, default='CONTCAR',
                        help="Path to input cif file, Default=./CONTCAR")
    parser.add_argument("-o", "--out_dir", nargs='?', type=str, default="./out",
                        help="Path to output dir, default = ./out")
    parser.add_argument("-t", "--type", nargs='?', type=str, default=None,
                        help="Type of diffusion atom, default = None")
    parser.add_argument("-l", "--path_length", nargs='?', type=int, default=5,
                        help="Maximum path length in Angstrom (INTEGER), default = 5")
    parser.add_argument("-n", "--npos", nargs='?', type=int, default=5,
                        help="Number of intermediate atomic positions in each path (INTEGER), default = 7")
    #npos=7           #Number of intermediate atomic positions in each path (INTEGER)
    #prec=5           #Accuracy of determining the length of diffusion parhs, =number of digits (INTEGER)
    #---------------------------------------------------------------------------------
    #Vicinity_rad=0.4 #Any structural atoms should be no closer to the path than $Vicinity_rad (REAL)
    parser.add_argument("-r", "--vicinity_radius", nargs='?', type=float, default=0.4,
                        help="Any structural atoms should be no closer to the path than $Vicinity_rad (REAL), default = 0.4")
    params = parser.parse_args()
    #input = args.input_file
    #out = args.out_dir
    return params

def debug_print(s):
    print(s)


def write_file(filename, string_list):
    if type(string_list) is list:
        with open(filename, "w") as outfile:
            outfile.write("\n".join(string_list))
        return
    with open(filename, "w") as outfile:
        outfile.write(string_list)


def Atoms_library(n):
    a = [
        'A1   H    H  0.25',
        'A2   He   He  0.00',
        'A3   Li   Li_sv  1.45  -3.81549558  2',
        'A4   Be   Be  1.05',
        'A5   B    B  0.85',
        'A6   C    C  0.70',
        'A7   N    N  0.65',
        'A8   O    O  0.60  -9.9048  2',
        'A9   F    F  0.50',
        'A10  Ne  Ne  0.00',
        'A11  Na  Na_pv  1.80  -2.62182219  2',
        'A12  Mg  Mg  1.50  -24.12013193  16',
        'A13  Al  Al  1.25',
        'A14  Si  Si  1.10',
        'A15  P   P  1.00',
        'A16  S   S  1.00',
        'A17  Cl  Cl  1.00',
        'A18  Ar  Ar  0.00',
        'A19  K   K_sv  2.20',
        'A20  Ca  Ca_sv  1.80  -7.99636959  4',
        'A21  Sc  Sc_sv  1.60',
        'A22  Ti  Ti_sv  1.40',
        'A23  V   V_sv  1.35',
        'A24  Cr  Cr_pv  1.40',
        'A25  Mn  Mn_pv  1.40',
        'A26  Fe  Fe  1.40',
        'A27  Co  Co  1.35',
        'A28  Ni  Ni  1.35',
        'A29  Cu  Cu  1.35',
        'A30  Zn  Zn  1.35  -2.52987428  2',
        'A31  Ga  Ga_d  1.30',
        'A32  Ge  Ge_d  1.25',
        'A33  As  As  1.15',
        'A34  Se  Se  1.15',
        'A35  Br  Br  1.15',
        'A36  Kr  Kr  0.00',
        'A37  Rb  Rb_sv  2.35',
        'A38  Sr  Sr_sv  2.00  -6.73330276  4',
        'A39  Y   Y_sv  1.80',
        'A40  Zr  Zr_sv  1.55',
        'A41  Nb  Nb_sv  1.45',
        'A42  Mo  Mo_sv  1.45',
        'A43  Tc  Tc_pv  1.35',
        'A44  Ru  Ru_pv  1.30',
        'A45  Rh  Rh_pv  1.35',
        'A46  Pd  Pd  1.40',
        'A47  Ag  Ag  1.60',
        'A48  Cd  Cd  1.55',
        'A49  In  In_d  1.55',
        'A50  Sn  Sn_d  1.45',
        'A51  Sb  Sb  1.45',
        'A52  Te  Te  1.40',
        'A53  I   I  1.40',
        'A54  Xe  Xe  0.00',
        'A55  Cs  Cs_sv  2.60',
        'A56  Ba  Ba_sv  2.15',
        'A57  La  La  1.95',
        'A58  Ce  Ce  1.85',
        'A59  Pr  Pr_3  1.85',
        'A60  Nd  Nd_3  1.85',
        'A61  Pm  Pm_3  1.85',
        'A62  Sm  Sm_3  1.85',
        'A63  Eu  Eu_2  1.85',
        'A64  Gd  Gd_3  1.80',
        'A65  Tb  Tb_3  1.75',
        'A66  Dy  Dy_3  1.75',
        'A67  Ho  Ho_3  1.75',
        'A68  Er  Er_3  1.75',
        'A69  Tm  Tm_3  1.75',
        'A70  Yb  Yb_2  1.75',
        'A71  Lu  Lu_3  1.75',
        'A72  Hf  Hf_pv  1.55',
        'A73  Ta  Ta_pv  1.45',
        'A74  W   W_pv  1.35',
        'A75  Re  Re  1.35',
        'A76  Os  Os  1.30',
        'A77  Ir  Ir  1.35',
        'A78  Pt  Pt  1.35',
        'A79  Au  Au  1.35',
        'A80  Hg  Hg  1.50',
        'A81  Tl  Tl_d  1.90',
        'A82  Pb  Pb_d  0.00',
        'A83  Bi  Bi_d  1.60',
        'A84  Po  Po_d  1.90',
        'A85  At  At_d  0.00',
        'A86  Rn  Rn  0.00',
        'A87  Fr  Fr_sv  0.00',
        'A88  Ra  Ra_sv  2.15',
        'A89  Ac  Ac  1.95',
        'A90  Th  Th  1.80',
        'A91  Pa  Pa  1.80',
        'A92  U   U  1.75',
        'A93  Np  Np  1.75',
        'A94  Pu  Pu  1.75',
        'A95  Am  Am  1.75',
        'A96  Cm  Cm  1.75',
        'A97  Bk  -',
        'A98  Cf  -',
        'A99  Es  -',
        'A100  Fm  -',
        ]
    r = a[n-1].split()
    return r[1]

# вставляем npos промежуточных точек между p1 и p2
def nebmake(p1, p2, npos):
    dx = []
    npos1 = npos + 1
    dx.append((p2[0] - p1[0]) / npos1)
    dx.append((p2[1] - p1[1]) / npos1)
    dx.append((p2[2] - p1[2]) / npos1)
    r = [ [p1[j]+i*dx[j] for j in [0,1,2]]
        for i in range(1,npos1)]
    return r

class ContcarFile():
    def __init__(self, filename, params):
        self.header_len = 8
        self.PATHS_txt = 'Created by PATHFINDER script v.3.0.5. Contact email: artkabanov@mail.ru\n'
        self.PATHS_txt += 'Date: '+ str(datetime.now())+"\n\n"
        self.params = params
        filename = params['input_file']
        self.filename = filename
        #absfn = os.path.abspath(filename)
        #dirn  = os.path.dirname(absfn)
        #psFilenamesList = glob.glob(dirn+'/POS*')
        self.MS = params['MS']
        self.all_lines = self.read()
        self.atom_names = self.all_lines[5].strip().split()
        self.atoms_nums_str = self.all_lines[6].strip().split() #Number of atoms
        self.atoms_nums_int = list(map(int,self.atoms_nums_str))

        a = [z[0]+z[1] for z in zip(self.atom_names, self.atoms_nums_str)]
        SYS = ' '.join(a)
        self.PATHS_txt += 'CHEMICAL SYSTEM: ' + SYS +"\n\n"#O96 Li56 La24 Zr16

        self.scale_factor = float((self.all_lines[1].split())[0])
        self.point1 = self.str2floatlist(2)
        self.point2 = self.str2floatlist(3)
        self.point3 = self.str2floatlist(4)
        self.calc_ABC()

        if not(self.MS in self.atom_names):
            print ('*  The type of atoms you have chosen ("{}") is absent   *'.format(self.MS))
            exit()
        iMS = self.atom_names.index(self.MS)
        self.indexMS = iMS
        self.atom_names_new = self.atom_names.copy()
        self.atom_names_new.pop(iMS)
        self.atom_names_new.append(self.MS)

        nums = self.atoms_nums_int.copy()
        self.countMS = nums.pop(iMS)
        self.atoms_nums_int_woMS = nums
        #numMS = self.atoms_nums_int[iMS]

        debug_print ("OK, lets rock")
        #get Ion coord
        pos = self.header_len #strings in header
        i = 0
        while i<iMS :
            pos += self.atoms_nums_int[i]
            i += 1
        max_pos = sum ( self.atoms_nums_int,0)

        #debug_print ("Coord for {}: pos from={} count={}  maxpos={}".format(MS, pos, num_of_atoms[iMS], max_pos))

        #poscar_MS = [ list(map(float,l.split())) for l in contcar[pos:pos+int(num_of_atoms[iMS])]]
        self.poscar_MS = [ self.str2floatlist(n) for n in range(pos, pos+self.atoms_nums_int[iMS]) ]
        self.find_pos_files()

        BODY_vasp = self.all_lines[8:pos]
        BODY_vasp2 = self.all_lines[pos+self.atoms_nums_int[iMS]:max_pos+self.header_len]
        BODY_vasp.extend(BODY_vasp2)
        self.body_vasp = BODY_vasp
        #print (poscar_MS[0])
        self.calc_path_dist()
        self.create_paths()
        return None

    def find_pos_files(self):
        absfn = os.path.abspath(self.filename)
        dirn  = os.path.dirname(absfn)
        psFilenamesList = glob.glob(dirn+'/POS_*')
        if len(psFilenamesList) < 1:
            return

        file1 = psFilenamesList[0]
        with open(file1) as file:
            pos_lines = [line.rstrip() for line in file]
        xyz = list(map(float,pos_lines[0].split()))
        if len(xyz) < 3:
            return
        found = False
        for pos in self.poscar_MS:
            d = self.dist([pos[0]-xyz[0],pos[1]-xyz[1],pos[2]-xyz[2]])
            if d < 0.00000001:
                found = True
            #print(d)
        if not found:
            return

        new_poscar_MS = []
        for f in psFilenamesList:
            x = re.search('.*POS_(\d+)',f)
            if len(x.groups()) < 1:
                continue
            pos_n = int(x.group(1))
            with open(f) as file:
                pos_lines = [list(map(float,line.rstrip().split())) for line in file]
                #pos_lines2 = list(map(append(pos_n),pos_lines))
                for l in pos_lines:
                    l.append(pos_n)
                new_poscar_MS.extend(pos_lines)
        print(len(new_poscar_MS))
        if len(new_poscar_MS) == len(self.poscar_MS):
            self.poscar_MS = new_poscar_MS
        print('ok')


    def calc_ABC(self):
        m = self.dist(self.point1)
        self.cos_a = [self.point1[0]/m, self.point1[1]/m, self.point1[2]/m]
        self.sin_a = [sqrt(self.point1[1]*self.point1[1] + self.point1[2]*self.point1[2])/m,
                 sqrt(self.point1[0]*self.point1[0] + self.point1[2]*self.point1[2])/m,
                 sqrt(self.point1[1]*self.point1[1] + self.point1[0]*self.point1[0])/m]
        m = self.dist(self.point2)
        self.cos_b = [self.point2[0]/m, self.point2[1]/m, self.point2[2]/m]
        self.sin_b = [sqrt(self.point2[1]*self.point2[1] + self.point2[2]*self.point2[2])/m,
                 sqrt(self.point2[0]*self.point2[0] + self.point2[2]*self.point2[2])/m,
                 sqrt(self.point2[1]*self.point2[1] + self.point2[0]*self.point2[0])/m]
        m = self.dist(self.point3)
        self.cos_c = [self.point3[0]/m, self.point3[1]/m, self.point3[2]/m]
        self.sin_c = [sqrt(self.point3[1]*self.point3[1] + self.point3[2]*self.point3[2])/m,
                 sqrt(self.point3[0]*self.point3[0] + self.point3[2]*self.point3[2])/m,
                 sqrt(self.point3[1]*self.point3[1] + self.point3[0]*self.point3[0])/m]
        sf2 = self.scale_factor*self.scale_factor
        self.A = self.dist(self.point1,sf2)
        self.B = self.dist(self.point2,sf2)
        self.C = self.dist(self.point3,sf2)
        debug_print("cos_a={}  b={}  c={}".format(self.cos_a,self.cos_b,self.cos_c))
        debug_print("sin_a={}  b={}  c={}".format(self.sin_a,self.sin_b,self.sin_c))
        debug_print("A={} -- B={} -- C={}".format(self.A, self.B, self.C))
        self.PATHS_txt += "CELL PARAMETERS: a = {:7.4f} (A)".format(self.A)+'\n'
        self.PATHS_txt += "                 b = {:7.4f} (A)".format(self.B)+'\n'
        self.PATHS_txt += "                 c = {:7.4f} (A)".format(self.C)+'\n\n'
        self.PATHS_txt += "Maximum path length = {} (A)".format(self.params['path_length'])+'\n\n'
        self.PATHS_txt += "="*72+'\n\n'
        '''
        CELL PARAMETERS: a= 13.208 (A)
                          b= 13.208 (A)
                          c= 12.672 (A)
        '''
        self._alpha = acos(self.cos_c[1]/self.dist(self.cos_c))*180/pi
        self._gamma = acos(self.cos_b[0]/self.dist(self.cos_b))*180/pi
        self._beta = acos(self.cos_c[0]/self.dist(self.cos_c))*180/pi

        self._alpha = acos((self.point2[0]*self.point3[0]+self.point2[1]*self.point3[1]+self.point2[2]*self.point3[2])/self.dist(self.point2)/self.dist(self.point3))*180/pi
        self._beta = acos((self.point1[0]*self.point3[0]+self.point1[1]*self.point3[1]+self.point1[2]*self.point3[2])/self.dist(self.point1)/self.dist(self.point3))*180/pi
        self._gamma = acos((self.point1[0]*self.point2[0]+self.point1[1]*self.point2[1]+self.point1[2]*self.point2[2])/self.dist(self.point1)/self.dist(self.point2))*180/pi

        return

    def dist(self,point,sf=1):
        return sqrt( point[0]*point[0]*sf + point[1]*point[1]*sf + point[2]*point[2]*sf )

    def str2floatlist(self,strnum):
        return list(map(float,self.all_lines[strnum].split()))

    def read(self):
        with open(self.filename) as file:
            all_lines = [line.rstrip() for line in file]
        return all_lines

    def calc_path_dist(self):
        lDist = []
        tDist = []
        A = self.A
        B = self.B
        C = self.C
        for j in range(0, self.countMS-1):
            '''
            a_j=`head -n $j POSCAR.$MS | tail -n 1 | awk '{print$1}'` #a of j-th atom
            b_j=`head -n $j POSCAR.$MS | tail -n 1 | awk '{print$2}'` #b of j-th atom
            c_j=`head -n $j POSCAR.$MS | tail -n 1 | awk '{print$3}'` #c of j-th atom
            s_i=`head -n $j POSCAR.$MS | tail -n 1 | awk '{print$4}'` #site of j-th atom
            '''
            #a_j = self.poscar_MS[j][0]
            #b_j = self.poscar_MS[j][1]
            #c_j = self.poscar_MS[j][2]
            p1 = self.poscar_MS[j]
            s_j = p1[3] if len(p1)>3 else 1 #TODO ???
            '''
            x_j=$A*$a_j*$cos_ax)+($B*$b_j*$cos_bx)+($C*$c_j*$cos_cx)
            y_j=$A*$a_j*$cos_ay)+($B*$b_j*$cos_by)+($C*$c_j*$cos_cy)
            z_j=$A*$a_j*$cos_az)+($B*$b_j*$cos_bz)+($C*$c_j*$cos_cz)
            '''
            x_j = (A*p1[0]*self.cos_a[0]) + (B*p1[1]*self.cos_b[0]) + (C*p1[2]*self.cos_c[0])
            y_j = (A*p1[0]*self.cos_a[1]) + (B*p1[1]*self.cos_b[1]) + (C*p1[2]*self.cos_c[1])
            z_j = (A*p1[0]*self.cos_a[2]) + (B*p1[1]*self.cos_b[2]) + (C*p1[2]*self.cos_c[2])

            for k in range(j+1,self.countMS):
                #a_k = poscar_MS[k][0]
                #b_k = poscar_MS[k][1]
                #c_k = poscar_MS[k][2]
                p2 = self.poscar_MS[k]
                s_k = p2[3] if len(p2)>3 else 1
                x_k = (A*p2[0]*self.cos_a[0]) + (B*p2[1]*self.cos_b[0]) + (C*p2[2]*self.cos_c[0])
                y_k = (A*p2[0]*self.cos_a[1]) + (B*p2[1]*self.cos_b[1]) + (C*p2[2]*self.cos_c[1])
                z_k = (A*p2[0]*self.cos_a[2]) + (B*p2[1]*self.cos_b[2]) + (C*p2[2]*self.cos_c[2])
                '''
                Dx=($x_k)-($x_j))^2"|bc -l`;
                Dy=($y_k)-($y_j))^2"|bc -l`;
                Dz=($z_k)-($z_j))^2"|bc -l`;
                Dist=`echo "scale=3; sqrt($Dx+$Dy+$Dz)"|bc -l`;
                Dist_r=`printf "%1.0f" $Dist; echo`;
                '''
                pointD = [(x_k - x_j),(y_k - y_j),(z_k - z_j)]
                #Dx = (x_k - x_j)*(x_k - x_j)
                #Dy = (y_k - y_j)*(y_k - y_j)
                #Dz = (z_k - z_j)*(z_k - z_j)
                Dist = self.dist(pointD)
                if Dist < self.params['path_length'] +0.5 :
                    d = "{:.4f} {}:{}".format(Dist,s_j,s_k)
                    lDist.append(d)
                    s = "{:=2} ({}) - {:=2} ({}) {:.4f}".format(j+1,s_j, k+1,s_k, Dist)
                    tDist.append( (d,s) )
                    #print (s)
        self.uniq_Dist = sorted(list(set(lDist)))
        self.paths_count = len(self.uniq_Dist)
        self.tDist = tDist

    def create_poscar_12(self,path_POSCAR,path_POSCAR_s,path_POSCAR_f,cur_subpath):
        # потом в конец вставляем стартовую
        POSCAR_start_tmp = path_POSCAR.copy()
        POSCAR_start_tmp.append(path_POSCAR_s)
        POSCAR_start_tmp = [
            ' '.join(
                map("{:.16f}".format,e)
            )
            for e in POSCAR_start_tmp]
        # ...и финишную
        POSCAR_finish_tmp = path_POSCAR.copy()
        POSCAR_finish_tmp.append(path_POSCAR_f)
        POSCAR_finish_tmp = [
            ' '.join(
                map("{:.16f}".format,e)
            )
            for e in POSCAR_finish_tmp]

        nn = self.atoms_nums_int_woMS.copy()
        nn.append(self.countMS-1)
        SPECIES_Ns = '  ' + '  '.join(list(map(str,nn)))

        # формируем файлы в подкаталоги 1/ 2/
        POSCAR_start = self.all_lines[:5]
        #cat ../../POSCAR.head ../../SPECIES_Ns.txt ../../BODY.vasp POSCAR.start.tmp > POSCAR.start
        POSCAR_start.append(' '.join(self.atom_names_new))
        POSCAR_start.append(SPECIES_Ns)
        POSCAR_start.append('Direct')
        POSCAR_start.extend(self.body_vasp)

        POSCAR_finish = POSCAR_start.copy()
        POSCAR_start.extend(POSCAR_start_tmp)
        # save to subdir 1/POSCAR
        write_file(cur_subpath+'/1/POSCAR',POSCAR_start)

        POSCAR_finish.extend(POSCAR_finish_tmp)
        # save to subdir 2/POSCAR
        write_file(cur_subpath+'/2/POSCAR',POSCAR_finish)

    def transform_coord(self, p):
        x = [0.0, 0.0, 0.0]
        x[0] = (self.A*p[0]*self.cos_a[0])+(self.B*p[1]*self.cos_b[0])+(self.C*p[2]*self.cos_c[0])  # Transform fractional
        x[1] = (self.A*p[0]*self.cos_a[1])+(self.B*p[1]*self.cos_b[1])+(self.C*p[2]*self.cos_c[1])  # coordinates in
        x[2] = (self.A*p[0]*self.cos_a[2])+(self.B*p[1]*self.cos_b[2])+(self.C*p[2]*self.cos_c[2])  # cartesian.
        return x

    def is_path_good(self, path_data):
        #body_float = self.body_vasp.strip().split()
        body_float = [ list(map(float,str.split())) for str in self.body_vasp ]
        path_is_good = True
        #for p in path_data:
        p = path_data[0]
        p_list = p.split()
        ATOM_START = int(p_list[0])
        ATOM_FIN = int(p_list[3])
        #path_POSCAR = self.poscar_MS.copy()
        Lf = self.poscar_MS[ATOM_FIN-1]
        Ls = self.poscar_MS[ATOM_START-1]
        x1 = self.transform_coord(Ls)
        x2 = self.transform_coord(Lf)
        #=====================================================
        #        Determination of directing vector (DV)
        #=====================================================
        DV = [ x2[0]-x1[0], x2[1]-x1[1], x2[2]-x1[2] ]
        L_DV = 0.00001 + self.dist(DV)# |DV|  0.00001 is added to avoid zero value
        #====================================================
        #                    Distance analysis
        #====================================================
        for latice_atom in body_float:
            #M = list(map(float, latice_atom.split()))
            M = self.transform_coord(latice_atom)

            #M1x=`echo "scale=6; (xM-x1)"|bc -l`;  # (M1)x
            #M1y=`echo "scale=6; (xM-y1)"|bc -l`;  # (M1)y
            #M1z=`echo "scale=6; (zM-z1)"|bc -l`;  # (M1)z
            #====================================================
            #                    Cross product of (DV)*(M1)
            #====================================================
            MxDV = [ DV[1]*(M[2]-x1[2])-(DV[2]*(M[1]-x1[1])),
                     DV[0]*(M[2]-x1[2])-(DV[2]*(M[0]-x1[0])),
                     DV[0]*(M[1]-x1[1])-(DV[1]*(M[0]-x1[0])) ]
            #ML1_DV_x = DVy*(zM-z1) - (DVz*(yM-y1))
            #ML1_DV_y = DVx*(zM-z1)-(DVz*(xM-x1))
            #ML1_DV_z = DVx*(xM-y1)-(DVy*(xM-x1))
            Area = self.dist(MxDV)
            #sqrt(ML1_DV_x*ML1_DV_x + ML1_DV_y*ML1_DV_y + ML1_DV_z*ML1_DV_z)
            Distance = 100*Area/L_DV
            #Distance=`printf "%1.0f" Distance; echo`;
            #ERROR_SUM += 1 if Distance<Vicinity_rad else 0
            if Distance < self.params['vicinity_rad_100'] :
            #if Distance < 60 :
                debug_print("BAD Path:  Dist={}  Vic_rad={}".format(Distance,self.params['vicinity_rad_100'] ))
                debug_print("Atom start  = {} {}".format(ATOM_START, str(Ls)))
                debug_print("Atom finish = {} {}".format(ATOM_FIN, str(Lf)))
                debug_print("latice atom = {}".format(str(latice_atom)))
                debug_print("DV = "+str(DV))
                debug_print("M  = "+str(M))
                debug_print("x1 = "+str(x1))
                path_is_good = False
                return False
                break
        return True


    def create_cif_header(self):
        txt = 'data_block_1\n'
        txt += "_audit_creation_method   'pathfinder v.3'"+'\n'
        txt += "_cell_length_a {:8.4f}".format(self.A) +'\n'
        txt += "_cell_length_b {:8.4f}".format(self.B) +'\n'
        txt += "_cell_length_c {:8.4f}".format(self.C) +'\n'
        txt += "_cell_angle_alpha {:8.4f}".format(self._alpha) +'\n'
        txt += "_cell_angle_beta  {:8.4f}".format(self._beta) +'\n'
        txt += "_cell_angle_gamma {:8.4f}".format(self._gamma) +'\n\n'
        txt += "_symmetry_space_group_name_H-M    'P 1'" +'\n'
        txt += "loop_" +'\n'
        txt += "_atom_site_label" +'\n'
        txt += "_atom_site_type_symbol" +'\n'
        txt += "_atom_site_fract_x" +'\n'
        txt += "_atom_site_fract_y" +'\n'
        txt += "_atom_site_fract_z" +'\n'
        txt += "_atom_site_occupancy" +'\n'
        return txt


    def add_path_info(self, path_num, path_dist, path_data):
        txt = '\n'
        txt += "PATH #{}".format(path_num) + '\n'
        txt += "LENGTH = {}".format(path_dist) + '\n\n'
        txt += "P1(Site)  P2(Site)  Dist\n"
        txt += '\n'.join(path_data)
        txt += '\n' + '-'*27 + '\n'
        self.PATHS_txt += txt

    def create_paths(self):
        Path_data =[]
        species_list = self.atoms_nums_int_woMS.copy()
        species_list.append(self.countMS)
        atom_names_full = self.atom_names_new.copy()
        path_num = 0
        #for path_num in range(1,self.paths_count+1):
        for n,path_dist in enumerate(self.uniq_Dist):
            debug_print("Path# {} -- Dist = {}".format(n,path_dist))
            #f = filter(lambda t: t[0] == self.uniq_Dist[path_num-1], self.tDist)
            f = filter(lambda t: t[0] == path_dist, self.tDist)
            path_data = [t[1] for t in f]
            if not self.is_path_good(path_data):
                debug_print("???")
                continue
            path_num += 1

            #mkdir PATH_<path_num>
            #file PATH_1/PATH_1.dat
            cur_path = self.params['out_dir'] + '/PATH_'+str(path_num)
            self.add_path_info(path_num, path_dist, path_data)
            os.mkdir(cur_path)
            path_n = []
            path_n.append("PATH #{}".format(path_num))
            path_n.append("LENGTH = {}".format(path_dist))
            path_n.append("")
            path_n.append("P1(Site)  P2(Site)  Dist")
            for p in path_data:
                path_n.append(p)
            #write_file('out/PATH_'+str(path_num)+'_PATH_'+str(path_num)+'.dat',path_n)
            write_file(cur_path+ '/PATH_'+str(path_num)+'.dat',path_n)
            # save to file

            #обработка
            Path_n_total = []

            # для каждого перехода создаем подкаталог PATH_1/PATH1.1
            cur_subpath = cur_path+'/PATH_'+str(path_num)+'.1'
            os.mkdir(cur_subpath)
            os.mkdir(cur_subpath+'/1')
            os.mkdir(cur_subpath+'/2')
            subpath_num = 0
            for p in path_data:
                #print(p)
                subpath_num += 1
                p_list = p.split()
                ATOM_START = int(p_list[0])
                ATOM_FIN = int(p_list[3])
                # выдергиваем две позиции
                path_POSCAR = self.poscar_MS.copy()
                path_POSCAR_f = path_POSCAR.pop(ATOM_FIN-1)
                path_POSCAR_s = path_POSCAR.pop(ATOM_START-1)
                tmp_filename = self.params['out_dir'] + '/PATH_'+str(path_num)+'_PATH_'+str(path_num)+'_'+str(subpath_num)+'_'
                if subpath_num == 1:
                    self.create_poscar_12(path_POSCAR,path_POSCAR_s,path_POSCAR_f,cur_subpath)

                #создаем промежутоные точки
                x = nebmake(path_POSCAR_s, path_POSCAR_f, self.params['npos'])
                Path_n_total.extend(x)

                nn = self.atoms_nums_int_woMS.copy()
                nn.append(self.countMS+len(x))
                SPECIES_Ns2 = '  ' + '  '.join(list(map(str,nn)))

                # save to PATH.vasp
                PATH_vasp = self.all_lines[:5]
                #cat ../../POSCAR.head ../../SPECIES_Ns.txt ../../BODY.vasp POSCAR.start.tmp > POSCAR.start
                PATH_vasp.append(' '.join(self.atom_names_new))
                PATH_vasp.append(SPECIES_Ns2)
                PATH_vasp.append('Direct')
                PATH_vasp.extend(self.body_vasp)
                x1 = ['  '+' '.join(map(str,l)) for l in self.poscar_MS]
                PATH_vasp.extend(x1)
                x2 = ['  '+' '.join(map(str,l)) for l in x]
                PATH_vasp.extend(x2)
                # save to PATH.vasp
                if subpath_num == 1:
                    write_file(cur_subpath+'/PATH.vasp',PATH_vasp)

                #print('end')

            #save to PATH_total.vasp
            PATH_total_vasp = list(map(lambda x: '  '+' '.join(map("{:.16f}".format,x)), Path_n_total))
            #write_file('out/PATH_'+str(path_num)+'_PATH_total.vasp',PATH_total_vasp)
            write_file(cur_path+'/PATH_total.vasp',PATH_total_vasp)

            Path_data.append(Path_n_total)

            #moving_xyz.append(moving_xyz_1[0])
            PATH_ELEMENT = Atoms_library(path_num)
            atom_names_full.append(PATH_ELEMENT)
            species_list.append(len(Path_n_total))
            #print('-')
        out = self.all_lines[:5]
        out.append('  '+'  '.join(atom_names_full))
        out.append('  '+'  '.join(map(str, species_list)))
        out.append('Direct')
        out.extend(self.body_vasp)
        out.extend(map(lambda x: '  '+' '.join(map("{:.16f}".format,x)), self.poscar_MS))
        for e in Path_data:
            out.extend(map(lambda x: '  '+' '.join(map("{:.16f}".format,x)), e))
        write_file(self.params['out_dir'] + '/FULL_MAP.vasp',out)
        write_file(self.params['out_dir'] + '/PATHS.txt',self.PATHS_txt)
        cif_out = self.create_cif_header()
        n = 7 #start position in out
        for i,a in enumerate(atom_names_full):
            str_s = a + '2   ' + a + '  '
            for j in range(species_list[i]):
                n += 1
                cif_out += str_s + out[n] + '  1\n'
        write_file(self.params['out_dir'] + '/FULL_MAP.vasp.cif',cif_out)

        return


def main():
    ###!!!
    ###MS=$1
    MS = "Zr"
    MS = "Li"
    #MS = sys.argv[2]

    #=================================================================================
    #
    #                                  MAIN PARAMETERS
    #
    #PATH_LENGTH = 3    #Maximum path length in Angstrom (INTEGER)
    #npos = 7           #Number of intermediate atomic positions in each path (INTEGER)
    prec = 5           #Accuracy of determining the length of diffusion parhs, =number of digits (INTEGER)
    #---------------------------------------------------------------------------------
    #Vicinity_rad=0.4 #Any structural atoms should be no closer to the path than $Vicinity_rad (REAL)

    params_arg = pf_parse_args()
    params = vars(params_arg)
    params['prec'] = 5
    params['vicinity_rad_100'] = int(100*params['vicinity_radius'])
    params['MS'] = params['type']

    try:
        os.mkdir(params['out_dir'])
    except Exception:
        pass
    '''
    input_file = 'CONTCAR'
    npos = 5
    out_dir = './out'
    path_length = 4
    type = None
    vicinity_radius = 0.5
    '''
    if params['input_file'].endswith(".cif"):
        from cif2cell.cif2cell_go import cif2cell_go
        cif2cell_go(params['input_file'],params['out_dir'])
        params['input_file'] = params['out_dir']+'/CONTCAR'
    #params = {}

    #cf = ContcarFile("infile.contcar",params)
    sys.argv[1] = params['input_file']
    cf = ContcarFile(sys.argv[1],params)

    #Vicinity_rad = int(100*Vicinity_rad)
    #debug_print("Vicinity_rad={}, NSPEC={}, SN={}".format(Vicinity_rad, NSPEC,SN))

    print ('END')


if __name__ == '__main__':
    #sys.tracebacklimit = 0
    main()
